Package: chimeraviz
Type: Package
Title: Visualization tools for gene fusions
Version: 1.36.0
Authors@R: c(
  person(
    given = "Stian",
    family = "Lågstad",
    email = "stianlagstad@gmail.com",
    role = c("aut", "cre")),
  person(
    given = "Sen",
    family = "Zhao",
    email = "Sen.Zhao@rr-research.no",
    role = "ctb"),
  person(
    given = "Andreas M.",
    family = "Hoff",
    email = "andreas.m.hoff@rr-research.no",
    role = "ctb"),
  person(
    given = "Bjarne",
    family = "Johannessen",
    email = "Bjarne.Johannessen@rr-research.no",
    role = "ctb"),
  person(
    given = "Ole Christian",
    family = "Lingjærde",
    email = "ole@ifi.uio.no",
    role = "ctb"),
  person(
    given = "Rolf",
    family = "Skotheim",
    email = "Rolf.I.Skotheim@rr-research.no",
    role = "ctb"))
Description: chimeraviz manages data from fusion gene finders and provides
    useful visualization tools.
License: Artistic-2.0
Imports: methods, grid, Rsamtools, GenomeInfoDb, GenomicAlignments,
        RColorBrewer, graphics, AnnotationDbi, RCircos, org.Hs.eg.db,
        org.Mm.eg.db, rmarkdown, graph, Rgraphviz, DT, plyr, dplyr,
        BiocStyle, checkmate, gtools, magick
Depends: Biostrings, GenomicRanges, IRanges, Gviz, S4Vectors,
        ensembldb, AnnotationFilter, data.table
Suggests: testthat, roxygen2, devtools, knitr, lintr
SystemRequirements: bowtie, samtools, and egrep are required for some
        functionalities
RoxygenNote: 7.1.1
VignetteBuilder: knitr
biocViews: Infrastructure, Alignment
Encoding: UTF-8
URL: https://github.com/stianlagstad/chimeraviz
BugReports: https://github.com/stianlagstad/chimeraviz/issues
git_url: https://git.bioconductor.org/packages/chimeraviz
git_branch: RELEASE_3_22
git_last_commit: 36b8261
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:08:46 UTC; biocbuild
Author: Stian Lågstad [aut, cre],
  Sen Zhao [ctb],
  Andreas M. Hoff [ctb],
  Bjarne Johannessen [ctb],
  Ole Christian Lingjærde [ctb],
  Rolf Skotheim [ctb]
Maintainer: Stian Lågstad <stianlagstad@gmail.com>
