# Generated by roxygen2: do not edit by hand

export("downstream_partner_gene<-")
export("partner_gene_ensembl_id<-")
export("upstream_partner_gene<-")
export(add_fusion_reads_alignment)
export(create_fusion_report)
export(decide_transcript_category)
export(down_shift)
export(downstream_partner_gene)
export(fetch_reads_from_fastq)
export(fusion_spanning_reads_count)
export(fusion_split_reads_count)
export(fusion_to_data_frame)
export(get_ensembl_ids)
export(get_fusion_by_chromosome)
export(get_fusion_by_gene_name)
export(get_fusion_by_id)
export(get_transcripts_ensembl_db)
export(import_aeron)
export(import_chimpipe)
export(import_defuse)
export(import_ericscript)
export(import_fusioncatcher)
export(import_fusionmap)
export(import_infusion)
export(import_jaffa)
export(import_oncofuse)
export(import_prada)
export(import_soapfuse)
export(import_squid)
export(import_starfusion)
export(partner_gene_ensembl_id)
export(partner_gene_junction_sequence)
export(plot_circle)
export(plot_fusion)
export(plot_fusion_reads)
export(plot_fusion_separate)
export(plot_fusion_together)
export(plot_fusion_transcript)
export(plot_fusion_transcript_with_protein_domain)
export(plot_fusion_transcripts_graph)
export(plot_transcripts)
export(select_transcript)
export(split_on_utr_and_add_feature)
export(upstream_partner_gene)
export(write_fusion_reference)
exportClasses(Fusion)
exportClasses(PartnerGene)
import(BiocStyle)
import(Biostrings)
import(GenomicRanges)
import(Gviz)
import(IRanges)
import(RCircos)
import(graphics)
import(methods)
import(org.Hs.eg.db)
import(org.Mm.eg.db)
importFrom(AnnotationFilter,GeneIdFilter)
importFrom(DT,datatable)
importFrom(S4Vectors,mcols)
importFrom(data.table,fread)
importFrom(ensembldb,EnsDb)
importFrom(ensembldb,exonsBy)
importFrom(plyr,ldply)
