#' chimeraviz: A package for working with and visualizing fusion genes.
#'
#' chimeraviz manages data from fusion gene finders and provides useful
#' visualization tools.
#'
#' @docType package
#' @name chimeraviz
#'
#' @import Biostrings GenomicRanges IRanges RCircos methods Gviz graphics
#' BiocStyle
NULL
#> NULL
